@@ -65,8 +65,8 @@ int paused = FALSE;
  * in which case you should set emission_id_wrapped and take care to check
  * an id isn't taken.
  **/
-uint32_t emission_id = 0;
-int      emission_id_wrapped = FALSE;
+unsigned int emission_id = 0;
+int          emission_id_wrapped = FALSE;
 
 /**
  * events:
@@ -163,33 +163,17 @@ event_copy (const void  *parent,
 		return NULL;
 
 	if (old_event->args) {
-		size_t   len;
-		char   **arg;
-
-		len = 0;
-		event->args = nih_str_array_new (event);
+		event->args = nih_str_array_copy (event, NULL,
+						  old_event->args);
 		if (! event->args)
 			goto error;
-
-		for (arg = old_event->args; arg && *arg; arg++)
-			if (! nih_str_array_add (&event->args, event,
-						 &len, *arg))
-				goto error;
 	}
 
 	if (old_event->env) {
-		size_t   len;
-		char   **env;
-
-		len = 0;
-		event->env = nih_str_array_new (event);
+		event->env = nih_str_array_copy (event, NULL,
+						 old_event->env);
 		if (! event->env)
 			goto error;
-
-		for (env = old_event->env; env && *env; env++)
-			if (! nih_str_array_add (&event->env, event,
-						 &len, *env))
-				goto error;
 	}
 
 	return event;
@@ -199,6 +183,7 @@ error:
 	return NULL;
 }
 
+
 /**
  * event_match:
  * @event1: first event,
@@ -252,22 +237,21 @@ event_match (Event *event1,
  * generated, at which point it becomes slightly less efficient.  If there
  * are currently 4 billion events being handled (!!) we lose the ability
  * to generate unique ids, and emit an error -- if we start seeing this in
- * the field, we can always increase the size to a 64-bit number or
- * something.
+ * the field, we can always to a larger type or something.
  *
  * Returns: next usable id.
  **/
-static inline uint32_t
+static inline unsigned int
 event_emit_next_id (void)
 {
-	uint32_t id;
+	unsigned int id;
 
 	/* If we've wrapped the emission_id counter, we can't just assume that
 	 * the current value isn't taken, we need to make sure that nothing
 	 * is using it first.
 	 */
 	if (emission_id_wrapped) {
-		uint32_t start_id = emission_id;
+		unsigned int start_id = emission_id;
 
 		while (event_emit_find_by_id (emission_id)) {
 			emission_id++;
@@ -276,7 +260,7 @@ event_emit_next_id (void)
 			 * we're currently handling 4 billion events.
 			 */
 			if (emission_id == start_id) {
-				nih_error (_("Emission id %zu is not unique"),
+				nih_error (_("Emission id %u is not unique"),
 					   emission_id);
 				break;
 			}
@@ -376,7 +360,7 @@ event_emit (const char       *name,
  * Returns: emission found or NULL if not found.
  **/
 EventEmission *
-event_emit_find_by_id (uint32_t id)
+event_emit_find_by_id (unsigned int id)
 {
 	EventEmission *emission;
 
@@ -525,13 +509,24 @@ event_finished (EventEmission *emission)
 
 		name = strrchr (emission->event.name, '/');
 		if ((! name) || strcmp (name, "/failed")) {
+			EventEmission *new_emission;
+
 			NIH_MUST (name = nih_sprintf (NULL, "%s/failed",
 						      emission->event.name));
-
-			event_emit (name, emission->event.args,
-				    emission->event.env);
-
+			new_emission = event_emit (name, NULL, NULL);
 			nih_free (name);
+
+			if (emission->event.args)
+				NIH_MUST (new_emission->event.args
+					  = nih_str_array_copy (
+						  new_emission, NULL,
+						  emission->event.args));
+
+			if (emission->event.env)
+				NIH_MUST (new_emission->event.env
+					  = nih_str_array_copy (
+						  new_emission, NULL,
+						  emission->event.env));
 		}
 	}
 
