@@ -153,7 +153,7 @@ typedef struct job_process {
 typedef struct job Job;
 struct job {
 	NihList         entry;
-	uint32_t        id;
+	unsigned int    id;
 
 	char           *name;
 	char           *description;
@@ -213,9 +213,9 @@ struct job {
 
 NIH_BEGIN_EXTERN
 
-uint32_t job_id;
-int      job_id_wrapped;
-NihHash *jobs;
+unsigned int  job_id;
+int           job_id_wrapped;
+NihHash      *jobs;
 
 
 void        job_init                  (void);
@@ -233,7 +233,7 @@ Job *       job_copy                  (const void *parent, const Job *old_job)
 
 Job *       job_find_by_name          (const char *name);
 Job *       job_find_by_pid           (pid_t pid, ProcessType *process);
-Job *       job_find_by_id            (uint32_t id);
+Job *       job_find_by_id            (unsigned int id);
 
 Job *       job_instance              (Job *job);
 void        job_change_goal           (Job *job, JobGoal goal,
