@@ -40,9 +40,10 @@
 
 
 /* Prototypes for static functions */
-static ssize_t          nih_config_block_end  (const char *file, size_t len,
+static int              nih_config_block_end  (const char *file, size_t len,
 					       size_t *lineno, size_t *pos,
-					       const char *type)
+					       const char *type,
+					       size_t *endpos)
 	__attribute__ ((warn_unused_result));
 static NihConfigStanza *nih_config_get_stanza (const char *name,
 					       NihConfigStanza *stanzas);
@@ -100,10 +101,12 @@ nih_config_has_token (const char *file,
  * @dest: destination to copy to,
  * @delim: characters to stop on,
  * @dequote: remove quotes and escapes.
+ * @toklen: pointer to store token length in.
  *
  * Parses a single token from @file which is stopped when any character
  * in @delim is encountered outside of a quoted string and not escaped
- * using a backslash.
+ * using a backslash.  The length of the parsed token is stored in @toklen
+ * if given.
  *
  * @file may be a memory mapped file, in which case @pos should be given
  * as the offset within and @len should be the length of the file as a
@@ -126,21 +129,20 @@ nih_config_has_token (const char *file,
  * If you also want quotes to be removed and escaped characters to be
  * replaced with the character itself, set @dequote to TRUE.
  *
- * Returns: the length of the token as it was/would be copied into @dest,
- * or negative value on raised error.
+ * Returns: zero on success, negative value on raised error.
  **/
-ssize_t
+int
 nih_config_token (const char *file,
 		  size_t      len,
 		  size_t     *pos,
 		  size_t     *lineno,
 		  char       *dest,
 		  const char *delim,
-		  int         dequote)
+		  int         dequote,
+		  size_t     *toklen)
 {
 	size_t  p, ws = 0, nlws = 0, qc = 0, i = 0;
-	ssize_t ret;
-	int     slash = FALSE, quote = 0, nl = FALSE;
+	int     slash = FALSE, quote = 0, nl = FALSE, ret = 0;
 
 	nih_assert (file != NULL);
 	nih_assert (delim != NULL);
@@ -254,13 +256,14 @@ nih_config_token (const char *file,
 	}
 
 
-	/* The return value is the length of the token with any newlines and
-	 * surrounding whitespace converted to a single character and any
-	 * trailing whitespace removed.
+	/* The token length we return is the length of the token with any
+	 * newlines and surrounding whitespace converted to a single
+	 * character and any trailing whitespace removed.
 	 *
 	 * The actual end of the text read is returned in *pos.
 	 */
-	ret = p - (pos ? *pos : 0) - ws - nlws - qc;
+	if (toklen)
+		*toklen = p - (pos ? *pos : 0) - ws - nlws - qc;
 
 finish:
 	if (pos)
@@ -316,21 +319,19 @@ nih_config_next_token (const void *parent,
 		       const char *delim,
 		       int         dequote)
 {
-	size_t   p, arg_start, arg_end;
-	ssize_t  arg_len;
-	char    *arg = NULL;
+	size_t  p, arg_start, arg_len, arg_end;
+	char   *arg = NULL;
 
 	nih_assert (file != NULL);
 
 	p = (pos ? *pos : 0);
 	arg_start = p;
-	arg_len = nih_config_token (file, len, &p, lineno,
-				    NULL, delim, dequote);
-	arg_end = p;
-
-	if (arg_len < 0) {
+	if (nih_config_token (file, len, &p, lineno, NULL, delim, dequote,
+			      &arg_len) < 0)
 		goto finish;
-	} else if (! arg_len) {
+
+	arg_end = p;
+	if (! arg_len) {
 		nih_error_raise (NIH_CONFIG_EXPECTED_TOKEN,
 				 _(NIH_CONFIG_EXPECTED_TOKEN_STR));
 		goto finish;
@@ -344,7 +345,7 @@ nih_config_next_token (const void *parent,
 		nih_return_system_error (NULL);
 
 	if (nih_config_token (file + arg_start, arg_end - arg_start, NULL,
-			      NULL, arg, delim, dequote) < 0)
+			      NULL, arg, delim, dequote, NULL) < 0)
 		goto finish;
 
 finish:
@@ -666,9 +667,8 @@ nih_config_parse_command (const void *parent,
 			  size_t     *pos,
 			  size_t     *lineno)
 {
-	char    *cmd = NULL;
-	size_t   p, cmd_start, cmd_end;
-	ssize_t  cmd_len;
+	char   *cmd = NULL;
+	size_t  p, cmd_start, cmd_len, cmd_end;
 
 	nih_assert (file != NULL);
 
@@ -677,13 +677,12 @@ nih_config_parse_command (const void *parent,
 	 */
 	p = (pos ? *pos : 0);
 	cmd_start = p;
-	cmd_len = nih_config_token (file, len, &p, lineno, NULL,
-				    NIH_CONFIG_CNL, FALSE);
-	cmd_end = p;
-
-	if (cmd_len < 0)
+	if (nih_config_token (file, len, &p, lineno, NULL,
+			      NIH_CONFIG_CNL, FALSE, &cmd_len) < 0)
 		goto finish;
 
+	cmd_end = p;
+
 	/* nih_config_token will eat up to the end of the file, a comment
 	 * or a newline; so this must always succeed.
 	 */
@@ -696,7 +695,7 @@ nih_config_parse_command (const void *parent,
 		nih_return_system_error (NULL);
 
 	if (nih_config_token (file + cmd_start, cmd_end - cmd_start, NULL,
-			      NULL, cmd, NIH_CONFIG_CNL, FALSE) < 0)
+			      NULL, cmd, NIH_CONFIG_CNL, FALSE, NULL) < 0)
 		goto finish;
 
 finish:
@@ -752,8 +751,7 @@ nih_config_parse_block (const void *parent,
 			const char *type)
 {
 	char    *block = NULL;
-	size_t   p, pp, sh_start, sh_len;
-	ssize_t  sh_end, ws;
+	size_t   p, pp, sh_start, sh_len, sh_end, ws;
 	int      lines;
 
 	nih_assert (file != NULL);
@@ -770,18 +768,18 @@ nih_config_parse_block (const void *parent,
 	 */
 	p = (pos ? *pos : 0);
 	sh_start = p;
-	sh_end = -1;
-	ws = -1;
+	sh_end = 0;
+	ws = 0;
 	lines = 0;
 
-	while ((sh_end = nih_config_block_end (file, len, &p,
-					       lineno, type)) < 0) {
+	while (! nih_config_block_end (file, len, &p, lineno, type, &sh_end)) {
 		size_t line_start;
 
 		lines++;
 		line_start = p;
-		if (ws < 0) {
-			/* Count initial whitespace */
+
+		if (lines == 1) {
+			/* Count whitespace on the first line */
 			while ((p < len) && strchr (NIH_CONFIG_WS, file[p]))
 				p++;
 
@@ -839,35 +837,103 @@ finish:
 	return block;
 }
 
+/**
+ * nih_config_skip_block:
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number,
+ * @type: block identifier,
+ * @endpos: pointer to end of block.
+ *
+ * Skips over a block of text from @file, stopping when the phrase
+ * "end @type" is encountered without any quotes or blackslash escaping
+ * within it.
+ *
+ * @file may be a memory mapped file, in which case @pos should be given
+ * as the offset within and @len should be the length of the file as a
+ * whole.
+ *
+ * If @pos is given then it will be used as the offset within @file to
+ * begin (otherwise the start is assumed), and will be updated to point
+ * past the end of the block and block marker or the end of the file.
+ *
+ * Either @file or @pos should point to the start of the block, after the
+ * opening stanza, rather than the start of the stanza that opens it.
+ *
+ * If @lineno is given it will be incremented each time a new line is
+ * discovered in the file.
+ *
+ * @endpos will be set to the end of the block and the start of the block
+ * marker, this is useful for determining the length of the block skipped,
+ * to parse it for example.
+ *
+ * Returns: zero on success, negative value on raised error.
+ **/
+int
+nih_config_skip_block (const char *file,
+		       size_t      len,
+		       size_t     *pos,
+		       size_t     *lineno,
+		       const char *type,
+		       size_t     *endpos)
+{
+	size_t p;
+	int    ret = 0;
+
+	nih_assert (file != NULL);
+	nih_assert (type != NULL);
+
+	p = (pos ? *pos : 0);
+
+	while (! nih_config_block_end (file, len, &p, lineno, type, endpos)) {
+		nih_config_next_line (file, len, &p, lineno);
+
+		if (p >= len) {
+			nih_error_raise (NIH_CONFIG_UNTERMINATED_BLOCK,
+					 _(NIH_CONFIG_UNTERMINATED_BLOCK_STR));
+			ret = -1;
+			goto finish;
+		}
+	}
+
+finish:
+	if (pos)
+		*pos = p;
+
+	return ret;
+}
+
 /**
  * nih_config_block_end:
  * @file: file or string to parse,
  * @len: length of @file,
  * @pos: offset within @file,
  * @lineno: line number,
- * @type: block identifier.
+ * @type: block identifier,
+ * @endpos: pointer to end of block.
  *
- * Determines whether the current line contains an end of block marker.
+ * Determines whether the current line contains an end of block marker,
+ * and if so, sets @endpos to the end  of the block.
  *
  * @file may be a memory mapped file, in which case @pos should be given
  * as the offset within and @len should be the length of the file.  @pos
- * will be updated to point past the end of the block or the end of the
- * file.
+ * will be updated to point past the end of the block and the end block
+ * marker or the end of the file.
  *
  * @lineno will be incremented each time a new line is discovered.
  *
- * Returns: index of block end (always the value of @pos at the time this
- * function was called) or -1 if it is not on this line.
+ * Returns: TRUE if at the end of the block, FALSE otherwise.
  **/
-static ssize_t
+static int
 nih_config_block_end (const char *file,
 		      size_t      len,
 		      size_t     *pos,
 		      size_t     *lineno,
-		      const char *type)
+		      const char *type,
+		      size_t     *endpos)
 {
-	size_t  p;
-	ssize_t end;
+	size_t p;
 
 	nih_assert (file != NULL);
 	nih_assert (pos != NULL);
@@ -883,11 +949,11 @@ nih_config_block_end (const char *file,
 	 * the need for whitespace immediately after)
 	 */
 	if ((len - p < 4) || strncmp (file + p, "end", 3))
-		return -1;
+		return FALSE;
 
 	/* Must be whitespace after */
 	if (! strchr (NIH_CONFIG_WS, file[p + 3]))
-		return -1;
+		return FALSE;
 
 	/* Find the second word */
 	p += 3;
@@ -897,7 +963,7 @@ nih_config_block_end (const char *file,
 	/* Check the second word */
 	if ((len - p < strlen (type))
 	    || strncmp (file + p, type, strlen (type)))
-		return -1;
+		return FALSE;
 
 	/* May be followed by whitespace */
 	p += strlen (type);
@@ -913,7 +979,7 @@ nih_config_block_end (const char *file,
 
 	/* Should be end of string, or a newline */
 	if ((p < len) && (file[p] != '\n'))
-		return -1;
+		return FALSE;
 
 	/* Point past the new line */
 	if (p < len) {
@@ -922,13 +988,14 @@ nih_config_block_end (const char *file,
 		p++;
 	}
 
-	/* Return the beginning of the line (which is the end of the script)
-	 * but update pos to point past this line.
+	/* Set endpos to the beginning of the line (which is the end of the
+	 * script) but update pos to point past this line.
 	 */
-	end = *pos;
+	if (endpos)
+		*endpos = *pos;
 	*pos = p;
 
-	return end;
+	return TRUE;
 }
 
 
@@ -987,9 +1054,6 @@ nih_config_get_stanza (const char      *name,
  * If @lineno is given it will be incremented each time a new line is
  * discovered in the file.
  *
- * A warning is output if no stanza is found, and the position is moved
- * past the end of the line.  This may result in inconsistent parsing.
- *
  * Returns: zero on success or negative value on raised error.
  **/
 int
@@ -1011,7 +1075,8 @@ nih_config_parse_stanza (const char      *file,
 	p = (pos ? *pos : 0);
 
 	/* Get the next dequoted argument from the file */
-	name = nih_config_next_arg (NULL, file, len, &p, lineno);
+	name = nih_config_next_token (NULL, file, len, &p, lineno,
+				      NIH_CONFIG_CNLWS, FALSE);
 	if (! name)
 		goto finish;
 
@@ -1020,15 +1085,15 @@ nih_config_parse_stanza (const char      *file,
 	if (! stanza) {
 		nih_error_raise (NIH_CONFIG_UNKNOWN_STANZA,
 				 _(NIH_CONFIG_UNKNOWN_STANZA_STR));
-		goto finish;
+		nih_free (name);
+		return -1;
 	}
 
+	nih_free (name);
+
 	ret = stanza->handler (data, stanza, file, len, &p, lineno);
 
 finish:
-	if (name)
-		nih_free (name);
-
 	if (pos)
 		*pos = p;
 
@@ -1147,19 +1212,11 @@ nih_config_parse (const char      *filename,
 	if (! file)
 		return -1;
 
-	if (len > SSIZE_MAX) {
-		nih_error_raise (NIH_CONFIG_TOO_LONG,
-				 _(NIH_CONFIG_TOO_LONG_STR));
-		ret = -1;
-		goto finish;
-	}
-
 	if (lineno)
 		*lineno = 1;
 
 	ret = nih_config_parse_file (file, len, pos, lineno, stanzas, data);
 
-finish:
 	if (nih_file_unmap ((void *)file, len) < 0)
 		return -1;
 
