@@ -83,6 +83,40 @@ nih_list_new (const void *parent)
 	return list;
 }
 
+/**
+ * nih_list_entry_new:
+ * @parent: parent of new list entry.
+ *
+ * Allocates a new list entry structure, leaving the caller to set the
+ * data of the entry.
+ *
+ * The structure is allocated using nih_alloc() so can be used as a context
+ * to other allocations.
+ *
+ * If @parent is not NULL, it should be a pointer to another allocated
+ * block which will be used as the parent for this block.  When @parent
+ * is freed, the returned string will be freed too.  If you have clean-up
+ * that would need to be run, you can assign a destructor function using
+ * the nih_alloc_set_destructor() function.
+ *
+ * Returns: the new list entry or NULL if the allocation failed.
+ **/
+NihListEntry *
+nih_list_entry_new (const void *parent)
+{
+	NihListEntry *list;
+
+	list = nih_new (parent, NihListEntry);
+	if (! list)
+		return NULL;
+
+	nih_list_init (&list->entry);
+
+	list->data = NULL;
+
+	return list;
+}
+
 
 /**
  * nih_list_add:
