@@ -2,7 +2,7 @@
  *
  * option.c - command-line argument and option parsing
  *
- * Copyright © 2006 Scott James Remnant <scott@netsplit.com>.
+ * Copyright © 2007 Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@
  * @parent: parent for new allocations,
  * @argc: number of arguments,
  * @argv: arguments array,
- * @options: lists of options,
+ * @options: list of options,
  * @nargs: number of non-option arguments,
  * @args: non-option arguments to return,
  * @arg: current argument index,
@@ -54,22 +54,21 @@
  * re-entrant.
  **/
 typedef struct nih_option_ctx {
-	void       *parent;
-	int         argc;
-	char      **argv;
-	NihOption  *options[3];
+	const void  *parent;
+	int          argc;
+	char       **argv;
+	NihOption   *options;
 
-	size_t      nargs;
-	char      **args;
+	size_t       nargs;
+	char       **args;
 
-	int         arg;
-	int         nonopt;
-	int         optend;
+	int          arg;
+	int          nonopt;
+	int          optend;
 } NihOptionCtx;
 
 
 /* Prototypes for static functions */
-static void        nih_option_add_arg     (NihOptionCtx *ctx);
 static NihOption * nih_option_get_short   (NihOptionCtx *ctx, int option);
 static int         nih_option_short       (NihOptionCtx *ctx);
 static NihOption * nih_option_get_long    (NihOptionCtx *ctx,
@@ -79,8 +78,9 @@ static int         nih_option_handle      (NihOptionCtx *ctx, NihOption *opt);
 static int         nih_option_handle_arg  (NihOptionCtx *ctx, NihOption *opt,
 					   const char *arg);
 static const char *nih_option_next_nonopt (NihOptionCtx *ctx);
+static void        nih_option_help        (NihOption *options);
 static void        nih_option_group_help  (NihOptionGroup *group,
-					   NihOption *options[],
+					   NihOption *options,
 					   NihOptionGroup **groups);
 
 
@@ -108,13 +108,41 @@ static NihOption default_options[] = {
 	NIH_OPTION_LAST
 };
 
+/**
+ * usage_stem:
+ *
+ * This string is prepended to the program's usage line if given.
+ **/
+static const char *usage_stem = NULL;
+
 /**
  * usage_string:
  *
- * This string is appended to the program help if given.
+ * This string is appended to the program's usage line if given.
  **/
 static const char *usage_string = NULL;
 
+/**
+ * synopsis_string:
+ *
+ * This string is output after the program's usage string if given.
+ **/
+static const char *synopsis_string = NULL;
+
+/**
+ * help_string:
+ *
+ * This string is output after the options if given.
+ **/
+static const char *help_string = NULL;
+
+/**
+ * footer_string:
+ *
+ * This string is output after the options and help if given.
+ **/
+static const char *footer_string = NULL;
+
 
 /**
  * nih_option_parser:
@@ -130,21 +158,31 @@ static const char *usage_string = NULL;
  * and arguments need not immediately follow the option that requires it.
  *
  * Remaining non-option arguments are placed into an array for processing
- * by the caller.  If @break_nonopt is %FALSE then the first non-option
+ * by the caller.  If @break_nonopt is FALSE then the first non-option
  * argument concludes option processing and all subsequent options are
  * considered to be ordinary arguments; this is most useful when the
  * first argument should be a command.
  *
+ * Both the array itself, and the array items are allocated with nih_alloc();
+ * the items are children of the array, so it is only necessary to call
+ * nih_free() on the array.
+ *
+ * If @parent is not NULL, it should be a pointer to another allocated
+ * block which will be used as the parent for this block.  When @parent
+ * is freed, the returned string will be freed too.  If you have clean-up
+ * that would need to be run, you can assign a destructor function using
+ * the nih_alloc_set_destructor() function.
+ *
  * Errors are handled by printing a message to standard error.
  *
- * Returns: non-option arguments array or %NULL on error.
+ * Returns: non-option arguments array or NULL on error.
  **/
 char **
-nih_option_parser (void      *parent,
-		   int        argc,
-		   char      *argv[],
-		   NihOption *options,
-		   int        break_nonopt)
+nih_option_parser (const void *parent,
+		   int         argc,
+		   char       *argv[],
+		   NihOption  *options,
+		   int         break_nonopt)
 {
 	NihOptionCtx ctx;
 
@@ -156,13 +194,10 @@ nih_option_parser (void      *parent,
 	ctx.argc = argc;
 	ctx.argv = argv;
 
-	ctx.options[0] = options;
-	ctx.options[1] = default_options;
-	ctx.options[2] = NULL;
+	ctx.options = nih_option_join (parent, options, default_options);
 
-	NIH_MUST (ctx.args = nih_alloc (parent, sizeof (char *)));
-	ctx.args[0] = NULL;
 	ctx.nargs = 0;
+	NIH_MUST (ctx.args = nih_str_array_new (parent));
 
 	ctx.nonopt = 0;
 	ctx.optend = 0;
@@ -175,7 +210,9 @@ nih_option_parser (void      *parent,
 		if ((arg[0] != '-') || (ctx.optend && ctx.arg > ctx.optend)) {
 			/* Not an option */
 			if (ctx.arg > ctx.nonopt) {
-				nih_option_add_arg (&ctx);
+				NIH_MUST (nih_str_array_add
+					  (&ctx.args, parent, &ctx.nargs,
+					   ctx.argv[ctx.arg]));
 				if (break_nonopt)
 					ctx.optend = ctx.arg;
 			}
@@ -196,36 +233,14 @@ nih_option_parser (void      *parent,
 		}
 	}
 
+	nih_free (ctx.options);
 	return ctx.args;
 error:
+	nih_free (ctx.options);
 	nih_free (ctx.args);
 	return NULL;
 }
 
-/**
- * nih_option_add_arg:
- * @ctx: parsing context.
- *
- * Add the current argument to the array of non-option arguments returned
- * to the caller.
- **/
-static void
-nih_option_add_arg (NihOptionCtx *ctx)
-{
-	char **new_args;
-	char  *arg;
-
-	nih_assert (ctx != NULL);
-
-	NIH_MUST (new_args = nih_realloc (ctx->args, ctx->parent,
-					  sizeof (char *) * (ctx->nargs + 2)));
-	NIH_MUST (arg = nih_strdup (new_args, ctx->argv[ctx->arg]));
-
-	ctx->args = new_args;
-	ctx->args[ctx->nargs++] = arg;
-	ctx->args[ctx->nargs] = NULL;
-}
-
 
 /**
  * nih_option_get_short:
@@ -236,21 +251,20 @@ nih_option_add_arg (NihOptionCtx *ctx)
  * exists with the short option '-' this is used instead if no specific
  * option is found.
  *
- * Returns; pointer to option, or %NULL if not found.
+ * Returns; pointer to option, or NULL if not found.
  **/
 static NihOption *
 nih_option_get_short (NihOptionCtx *ctx,
 		      int           option)
 {
-	NihOption *opt, **opts, *catch = NULL;
+	NihOption *opt, *catch = NULL;
 
-	for (opts = ctx->options; *opts != NULL; opts++) {
-		for (opt = *opts; (opt->option || opt->long_option); opt++) {
-			if (opt->option == option)
-				return opt;
-			if (opt->option == '-')
-				catch = opt;
-		}
+	for (opt = ctx->options; (opt->option || opt->long_option); opt++) {
+		if (opt->option == '-')
+			catch = opt;
+
+		if (opt->option == option)
+			return opt;
 	}
 
 	return catch;
@@ -318,29 +332,27 @@ nih_option_short (NihOptionCtx *ctx)
  * the first @len characters will be read.  If an option named "--" exists
  * then it is used if no other option could be found.
  *
- * Returns; pointer to option, or %NULL if not found.
+ * Returns; pointer to option, or NULL if not found.
  **/
 static NihOption *
 nih_option_get_long (NihOptionCtx *ctx,
 		     const char   *option,
 		     size_t        len)
 {
-	NihOption *opt, **opts, *catch = NULL;
+	NihOption *opt, *catch = NULL;
 
-	for (opts = ctx->options; *opts != NULL; opts++) {
-		for (opt = *opts; (opt->option || opt->long_option); opt++) {
-			if (! opt->long_option)
-				continue;
+	for (opt = ctx->options; (opt->option || opt->long_option); opt++) {
+		if (! opt->long_option)
+			continue;
 
-			if (! strcmp (opt->long_option, "--"))
-				catch = opt;
+		if (! strcmp (opt->long_option, "--"))
+			catch = opt;
 
-			if (strlen (opt->long_option) > len)
-				continue;
+		if (strlen (opt->long_option) > len)
+			continue;
 
-			if (! strncmp (option, opt->long_option, len))
-				return opt;
-		}
+		if (! strncmp (option, opt->long_option, len))
+			return opt;
 	}
 
 	return catch;
@@ -415,9 +427,9 @@ nih_option_long (NihOptionCtx *ctx)
  *
  * Handle an option which either does not take an argument, or should
  * take the next non-option argument from the command-line.  For options
- * with arguments, this calls #nih_option_handle_arg; for those without,
+ * with arguments, this calls nih_option_handle_arg(); for those without,
  * this calls the setter function or treats the value member as a pointer
- * to an int to store %TRUE in.
+ * to an int to store TRUE in.
  *
  * Returns: zero on success, negative value on invalid option.
  **/
@@ -432,11 +444,15 @@ nih_option_handle (NihOptionCtx *ctx,
 	if (opt->long_option && (! strcmp (opt->long_option, "help"))) {
 		/* --help */
 		nih_option_help (ctx->options);
+		nih_free (ctx->options);
+		nih_free (ctx->args);
 		exit (0);
 	} else if (opt->long_option
 		   && (! strcmp (opt->long_option, "version"))) {
 		/* --version */
 		nih_main_version ();
+		nih_free (ctx->options);
+		nih_free (ctx->args);
 		exit (0);
 	}
 
@@ -508,7 +524,7 @@ nih_option_handle_arg (NihOptionCtx *ctx,
  * that is not an option.  Updates the nonopt member of @ctx to point
  * at the option used.
  *
- * Returns: next non-option argument or %NULL if none remain.
+ * Returns: next non-option argument or NULL if none remain.
  **/
 static const char *
 nih_option_next_nonopt (NihOptionCtx *ctx)
@@ -536,6 +552,58 @@ nih_option_next_nonopt (NihOptionCtx *ctx)
 }
 
 
+/**
+ * nih_option_join:
+ * @parent: parent for new allocation,
+ * @a: first option array,
+ * @b: second option array.
+ *
+ * Joins the two option arrays together to produce a combined array containing
+ * the options from @a followed by the options from @b.
+ *
+ * The new list is allocated with nih_alloc(), but the members are just
+ * copied in from @a and @b including any pointers therein.  Freeing the
+ * new array with nih_free() is entirely safe.
+ *
+ * If @parent is not NULL, it should be a pointer to another allocated
+ * block which will be used as the parent for this block.  When @parent
+ * is freed, the returned string will be freed too.  If you have clean-up
+ * that would need to be run, you can assign a destructor function using
+ * the nih_alloc_set_destructor() function.
+ *
+ * Returns: combined option array.
+ **/
+NihOption *
+nih_option_join (const void *parent,
+		 NihOption  *a,
+		 NihOption  *b)
+{
+	NihOption *opt, *opts;
+	size_t     alen = 0, blen = 0;
+
+	nih_assert (a != NULL);
+	nih_assert (b != NULL);
+
+	/* Count options in first list */
+	for (opt = a; (opt->option || opt->long_option); opt++)
+		alen++;
+
+	/* Count options in second list */
+	for (opt = b; (opt->option || opt->long_option); opt++)
+		blen++;
+
+	/* Allocate combined list */
+	NIH_MUST (opts = nih_alloc (parent,
+				    sizeof (NihOption) * (alen + blen + 1)));
+
+	/* Copy options, making sure to copy the last option from b */
+	memcpy (opts, a, sizeof (NihOption) * alen);
+	memcpy (opts + alen, b, sizeof (NihOption) * (blen + 1));
+
+	return opts;
+}
+
+
 /**
  * nih_option_count:
  * @option: NihOption invoked,
@@ -545,7 +613,7 @@ nih_option_next_nonopt (NihOptionCtx *ctx)
  * of times they are placed on the command line.
  *
  * The value member of @option must be a pointer to an integer variable,
- * the arg_name member must be %NULL.
+ * the arg_name member must be NULL.
  *
  * Returns: always returns zero.
  **/
@@ -565,6 +633,43 @@ nih_option_count (NihOption  *option,
 	return 0;
 }
 
+/**
+ * nih_option_int:
+ * @option: NihOption invoked,
+ * @arg: argument to parse.
+ *
+ * This option setter may be used to parse an integer from the command line
+ * and store it in the value member of @option, which must be a pointer to
+ * an integer variable.
+ *
+ * The arg_name member of @option must not be NULL.
+ *
+ * Returns: zero on success, non-zero on error.
+ **/
+int
+nih_option_int (NihOption  *option,
+		const char *arg)
+{
+	char *endptr;
+	int  *value;
+
+	nih_assert (option != NULL);
+	nih_assert (option->value != NULL);
+	nih_assert (arg != NULL);
+
+	value = (int *)option->value;
+	*value = strtol (arg, &endptr, 10);
+
+	if (*endptr) {
+		fprintf (stderr, _("%s: illegal argument: %s\n"),
+			 program_name, arg);
+		nih_main_suggest_help ();
+		return -1;
+	}
+
+	return 0;
+}
+
 
 /**
  * nih_option_quiet:
@@ -633,32 +738,94 @@ nih_option_debug (NihOption  *option,
 }
 
 
+/**
+ * nih_option_set_usage_stem:
+ * @usage: usage stem.
+ *
+ * Set the usage stem prepended to the program usage line in the help output,
+ * this should be a static translated string.
+ *
+ * The string should not be terminated with a newline.
+ **/
+void
+nih_option_set_usage_stem (const char *usage)
+{
+	usage_stem = usage;
+}
+
 /**
  * nih_option_set_usage:
  * @usage: usage string.
  *
- * Set the usage string appended to the program help output, this should
- * be a static translated string.
+ * Set the usage string appended to the program usage line in the help output,
+ * this should be a static translated string.
+ *
+ * The string should not be terminated with a newline.
  **/
 void
 nih_option_set_usage (const char *usage)
 {
-	nih_assert (usage != NULL);
-
 	usage_string = usage;
 }
 
+/**
+ * nih_option_set_synopsis:
+ * @synopsis: synopsis string.
+ *
+ * Set the synopsis string, shown after the program usage line in the help
+ * output.  This should be a static translated string.  It will be
+ * automatically wrapped to the screen width.
+ *
+ * The string should not be terminated with a newline.
+ **/
+void
+nih_option_set_synopsis (const char *synopsis)
+{
+	synopsis_string = synopsis;
+}
+
+/**
+ * nih_option_set_help:
+ * @help: help string.
+ *
+ * Set the help string, this is displayed after the options in the help
+ * output.  This should be a static translated string.  It will be
+ * automatically wrapped to the screen width.
+ *
+ * The string should not be terminated with a newline.
+ **/
+void
+nih_option_set_help (const char *help)
+{
+	help_string = help;
+}
+
+/**
+ * nih_option_set_footer:
+ * @footer: footer string.
+ *
+ * Set the footer string, this is displayed after the options and help
+ * text in the output.  This should be a static translated string.
+ *
+ * The string should not be terminated with a newline.
+ **/
+void
+nih_option_set_footer (const char *footer)
+{
+	footer_string = footer;
+}
+
 /**
  * nih_option_help:
- * @options: program options lists.
+ * @options: program options list.
  *
  * Output a description of the program's options to standard output
  * grouped by the group member of the option.
  **/
-void
-nih_option_help (NihOption *options[])
+static void
+nih_option_help (NihOption *options)
 {
-	NihOption       *opt, **opts;
+	NihOption       *opt;
 	NihOptionGroup **groups;
 	size_t           group, ngroups;
 	int              other = FALSE;
@@ -669,37 +836,50 @@ nih_option_help (NihOption *options[])
 	ngroups = 0;
 
 	/* Count the number of option groups */
-	for (opts = options; *opts != NULL; opts++) {
-		for (opt = *opts; (opt->option || opt->long_option); opt++) {
-			NihOptionGroup **new_groups;
+	for (opt = options; (opt->option || opt->long_option); opt++) {
+		NihOptionGroup **new_groups;
 
-			if (! opt->group) {
-				other = TRUE;
-				continue;
-			}
-
-			for (group = 0; group < ngroups; group++) {
-				if (groups[group] == opt->group)
-					break;
-			}
-
-			if (group < ngroups)
-				continue;
-
-			NIH_MUST (new_groups = nih_realloc (
-					  groups, NULL,
-					  (sizeof (NihOptionGroup *)
-					   * (ngroups + 1))));
-			groups = new_groups;
-			groups[ngroups++] = opt->group;
+		if (! opt->group) {
+			other = TRUE;
+			continue;
 		}
+
+		for (group = 0; group < ngroups; group++) {
+			if (groups[group] == opt->group)
+				break;
+		}
+
+		if (group < ngroups)
+			continue;
+
+		NIH_MUST (new_groups = nih_realloc (groups, NULL,
+						    (sizeof (NihOptionGroup *)
+						     * (ngroups + 1))));
+		groups = new_groups;
+		groups[ngroups++] = opt->group;
 	}
 
-	printf ("%s: %s [OPTION]...", _("Usage"), program_name);
+	printf ("%s: %s", _("Usage"), program_name);
+	if (usage_stem) {
+		printf (" %s", usage_stem);
+	} else {
+		printf (" %s", _("[OPTION]..."));
+	}
 	if (usage_string)
 		printf (" %s", usage_string);
 	printf ("\n");
 
+	/* Wrap the synopsis to the screen width */
+	if (synopsis_string) {
+		char *str;
+
+		NIH_MUST (str = nih_str_screen_wrap (NULL, synopsis_string,
+						     0, 0));
+		printf ("%s\n", str);
+		nih_free (str);
+	}
+	printf ("\n");
+
 	/* Iterate the option groups we found in order, and display
 	 * only their options
 	 */
@@ -710,6 +890,26 @@ nih_option_help (NihOption *options[])
 	if (other)
 		nih_option_group_help (NULL, options, groups);
 
+	/* Wrap the help to the screen width */
+	if (help_string) {
+		char *str;
+
+		NIH_MUST (str = nih_str_screen_wrap (NULL, help_string, 0, 0));
+		printf ("%s\n", str);
+		nih_free (str);
+
+		if (package_bugreport || footer_string)
+			printf ("\n");
+	}
+
+	/* Append the footer */
+	if (footer_string) {
+		printf ("%s\n", footer_string);
+
+		if (package_bugreport)
+			printf ("\n");
+	}
+
 	/* Append the bug report address */
 	if (package_bugreport)
 		printf (_("Report bugs to <%s>\n"), package_bugreport);
@@ -721,17 +921,19 @@ nih_option_help (NihOption *options[])
 /**
  * nih_option_group_help:
  * @group: group to display,
- * @options: program options lists.
+ * @options: program options list,
+ * @groups: all groups.
  *
  * Output a list of the program's options in the given option group to
  * standard output.
  **/
 static void
 nih_option_group_help (NihOptionGroup  *group,
-		       NihOption       *options[],
+		       NihOption       *options,
 		       NihOptionGroup **groups)
 {
-	NihOption *opt, **opts;
+	NihOption *opt;
+	size_t     width;
 
 	nih_assert (options != NULL);
 
@@ -743,92 +945,91 @@ nih_option_group_help (NihOptionGroup  *group,
 		printf (_("Options:\n"));
 	}
 
-	for (opts = options; *opts != NULL; opts++) {
-		for (opt = *opts; (opt->option || opt->long_option); opt++) {
-			char   *str, *ptr;
-			size_t  len = 0;
+	width = MAX (nih_str_screen_width (), 50) - 31;
 
-			if (opt->group != group)
-				continue;
+	for (opt = options; (opt->option || opt->long_option); opt++) {
+		char   *str, *ptr;
+		size_t  len = 0;
 
-			if (! opt->help)
-				continue;
+		if (opt->group != group)
+			continue;
 
-			/* Indent by two spaces */
-			printf ("  ");
+		if (! opt->help)
+			continue;
+
+		/* Indent by two spaces */
+		printf ("  ");
+		len += 2;
+
+		/* Display the short option */
+		if (opt->option) {
+			printf ("-%c", opt->option);
 			len += 2;
 
-			/* Display the short option */
-			if (opt->option) {
-				printf ("-%c", opt->option);
-				len += 2;
-
-				/* Seperate short and long option, or
-				 * give the argument name
-				 */
-				if (opt->long_option) {
-					printf (", ");
-					len += 2;
-				} else if (opt->arg_name) {
-					printf (" %s", opt->arg_name);
-					len += strlen (opt->arg_name) + 1;
-				}
-			} else {
-				/* Make all long options the same indent
-				 * whether or not there's a short one
-				 */
-				printf ("    ");
-				len += 4;
-			}
-
-			/* Display the long option */
-			if (opt->long_option) {
-				printf ("--%s", opt->long_option);
-				len += strlen (opt->long_option) + 2;
-
-				/* With the argument name */
-				if (opt->arg_name) {
-					printf ("=%s", opt->arg_name);
-					len += strlen (opt->arg_name) + 1;
-				}
-			}
-
-			/* Format the help string to fit in the latter
-			 * half of the screen
+			/* Seperate short and long option, or
+			 * give the argument name
 			 */
-			NIH_MUST (str = nih_str_wrap (NULL, opt->help,
-						      50, 0, 2));
+			if (opt->long_option) {
+				printf (", ");
+				len += 2;
+			} else if (opt->arg_name) {
+				printf (" %s", opt->arg_name);
+				len += strlen (opt->arg_name) + 1;
+			}
+		} else {
+			/* Make all long options the same indent
+			 * whether or not there's a short one
+			 */
+			printf ("    ");
+			len += 4;
+		}
 
-			/* Write the description to the screen */
-			ptr = str;
-			while (ptr && *ptr) {
-				size_t linelen;
+		/* Display the long option */
+		if (opt->long_option) {
+			printf ("--%s", opt->long_option);
+			len += strlen (opt->long_option) + 2;
 
-				/* Not enough room on this line */
-				if (len > 28) {
-					printf ("\n");
-					len = 0;
-				}
+			/* With the argument name */
+			if (opt->arg_name) {
+				printf ("=%s", opt->arg_name);
+				len += strlen (opt->arg_name) + 1;
+			}
+		}
 
-				/* Pad line up to the right column */
-				while (len < 30) {
-					printf (" ");
-					len++;
-				}
+		/* Format the help string to fit in the latter
+		 * half of the screen
+		 */
+		NIH_MUST (str = nih_str_wrap (NULL, opt->help, width, 0, 2));
 
-				/* Output the line up until the next line */
-				linelen = strcspn (ptr, "\n");
-				printf ("%.*s\n", (int)linelen, ptr);
+		/* Write the description to the screen */
+		ptr = str;
+		while (ptr && *ptr) {
+			size_t linelen;
+
+			/* Not enough room on this line */
+			if (len > 28) {
+				printf ("\n");
 				len = 0;
-
-				/* Skip to the next line */
-				ptr += linelen;
-				if (*ptr == '\n')
-					ptr++;
 			}
 
-			nih_free (str);
+			/* Pad line up to the right column */
+			while (len < 30) {
+				printf (" ");
+				len++;
+			}
+
+			/* Output the line up until the next line */
+			linelen = strcspn (ptr, "\n");
+			printf ("%.*s\n", (int)linelen, ptr);
+			len = 0;
+
+			/* Skip to the next line */
+			ptr += linelen;
+			if (*ptr == '\n')
+				ptr++;
 		}
+
+		nih_free (str);
 	}
 
 	printf ("\n");
