@@ -43,6 +43,7 @@ char * nih_strndup          (const void *parent, const char *str, size_t len)
 char **nih_str_split        (const void *parent, const char *str,
 			     const char *delim, int repeat)
 	__attribute__ ((warn_unused_result, malloc));
+
 char **nih_str_array_new    (const void *parent)
 	__attribute__ ((warn_unused_result, malloc));
 char **nih_str_array_add    (char ***array, const void *parent, size_t *len,
@@ -54,6 +55,13 @@ char **nih_str_array_addn   (char ***array, const void *parent, size_t *len,
 char **nih_str_array_addp   (char ***array, const void *parent, size_t *len,
 			     void *ptr)
 	__attribute__ ((warn_unused_result, malloc));
+char **nih_str_array_copy   (const void *parent, size_t *len,
+			     char * const *array)
+	__attribute__ ((warn_unused_result, malloc));
+char **nih_str_array_append (char ***array, const void *parent, size_t *len,
+			     char * const *args)
+	__attribute__ ((warn_unused_result, malloc));
+
 void   nih_strv_free        (char **strv);
 
 char * nih_str_wrap         (const void *parent, const char *str, size_t len,
