@@ -102,8 +102,8 @@ void
 test_token (void)
 {
 	char      buf[1024], dest[1024];
-	size_t    pos, lineno;
-	ssize_t   ret;
+	size_t    pos, lineno, len;
+	int       ret;
 	NihError *err;
 
 	TEST_FUNCTION ("nih_config_token");
@@ -117,10 +117,12 @@ test_token (void)
 	strcpy (buf, "this is a test");
 	pos = 0;
 
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 4);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 4);
 	TEST_EQ (pos, 4);
 
 
@@ -130,10 +132,12 @@ test_token (void)
 	TEST_FEATURE ("with token filling string");
 	strcpy (buf, "wibble");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 6);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 6);
 	TEST_EQ (pos, 6);
 
 
@@ -143,9 +147,9 @@ test_token (void)
 	TEST_FEATURE ("with token to extract");
 	strcpy (buf, "this is a test");
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", FALSE);
+				dest, " ", FALSE, NULL);
 
-	TEST_EQ (ret, 4);
+	TEST_EQ (ret, 0);
 	TEST_EQ_STR (dest, "this");
 
 
@@ -154,10 +158,12 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with token inside string");
 	pos = 5;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 2);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 2);
 	TEST_EQ (pos, 7);
 
 
@@ -168,10 +174,12 @@ test_token (void)
 	TEST_FEATURE ("with double quotes inside token");
 	strcpy (buf, "\"this is a\" test");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ (pos, 11);
 
 
@@ -179,10 +187,12 @@ test_token (void)
 	 * quotes, we should still get those.
 	 */
 	TEST_FEATURE ("with double quotes around token to extract");
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", FALSE);
+				dest, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ_STR (dest, "\"this is a\"");
 
 
@@ -191,10 +201,12 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with double quotes and dequoting");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", TRUE);
+				NULL, " ", TRUE, &len);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 9);
 	TEST_EQ (pos, 11);
 
 
@@ -203,9 +215,9 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with double quotes and extract with dequoting");
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", TRUE);
+				dest, " ", TRUE, NULL);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
 	TEST_EQ_STR (dest, "this is a");
 
 
@@ -216,10 +228,12 @@ test_token (void)
 	TEST_FEATURE ("with single quotes inside token");
 	strcpy (buf, "\'this is a\' test");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ (pos, 11);
 
 
@@ -230,10 +244,12 @@ test_token (void)
 	TEST_FEATURE ("with escaped spaces inside token");
 	strcpy (buf, "this\\ is\\ a test");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ (pos, 11);
 
 
@@ -242,9 +258,10 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with escaped spaces within extracted token");
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", FALSE);
+				dest, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ_STR (dest, "this\\ is\\ a");
 
 
@@ -253,10 +270,12 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with escaped spaces inside token and dequoting");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", TRUE);
+				NULL, " ", TRUE, &len);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 9);
 	TEST_EQ (pos, 11);
 
 
@@ -264,10 +283,12 @@ test_token (void)
 	 * around the delimiter, while removing them.
 	 */
 	TEST_FEATURE ("with escaped spaces within extracted dequoted token");
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", TRUE);
+				dest, " ", TRUE, &len);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 9);
 	TEST_EQ_STR (dest, "this is a");
 
 
@@ -278,10 +299,12 @@ test_token (void)
 	strcpy (buf, "\"this is \n a\" test");
 	pos = 0;
 	lineno = 1;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ (pos, 13);
 	TEST_EQ (lineno, 2);
 
@@ -290,10 +313,12 @@ test_token (void)
 	 * string only returns a single space for the newline.
 	 */
 	TEST_FEATURE ("with newline inside extracted quoted string");
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, " ", FALSE);
+				dest, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ_STR (dest, "\"this is a\"");
 
 
@@ -303,10 +328,12 @@ test_token (void)
 	TEST_FEATURE ("with newline inside quoted string and lineno set");
 	pos = 0;
 	lineno = 1;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
-	TEST_EQ (ret, 11);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 11);
 	TEST_EQ (pos, 13);
 	TEST_EQ (lineno, 2);
 
@@ -318,10 +345,12 @@ test_token (void)
 	strcpy (buf, "this \\\n is a:test");
 	pos = 0;
 	lineno = 1;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, ":", FALSE);
+				NULL, ":", FALSE, &len);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 9);
 	TEST_EQ (pos, 12);
 	TEST_EQ (lineno, 2);
 
@@ -331,9 +360,9 @@ test_token (void)
 	 */
 	TEST_FEATURE ("with escaped newline inside extracted string");
 	ret = nih_config_token (buf, strlen (buf), NULL, NULL,
-				dest, ":", FALSE);
+				dest, ":", FALSE, NULL);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
 	TEST_EQ_STR (dest, "this is a");
 
 
@@ -343,10 +372,12 @@ test_token (void)
 	TEST_FEATURE ("with escaped newline inside string and lineno set");
 	pos = 0;
 	lineno = 1;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, ":", FALSE);
+				NULL, ":", FALSE, &len);
 
-	TEST_EQ (ret, 9);
+	TEST_EQ (ret, 0);
+	TEST_EQ (len, 9);
 	TEST_EQ (pos, 12);
 	TEST_EQ (lineno, 2);
 
@@ -357,10 +388,11 @@ test_token (void)
 	TEST_FEATURE ("with slash at end of string");
 	strcpy (buf, "wibble\\");
 	pos = 0;
+	len = 0;
 	lineno = 1;
 
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, NULL);
 
 	TEST_LT (ret, 0);
 	TEST_EQ (pos, 7);
@@ -380,7 +412,7 @@ test_token (void)
 	lineno = 1;
 
 	ret = nih_config_token (buf, strlen (buf), &pos, &lineno,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, NULL);
 
 	TEST_LT (ret, 0);
 	TEST_EQ (pos, 8);
@@ -397,10 +429,12 @@ test_token (void)
 	TEST_FEATURE ("with empty token");
 	strcpy (buf, " wibble");
 	pos = 0;
+	len = 0;
 	ret = nih_config_token (buf, strlen (buf), &pos, NULL,
-				NULL, " ", FALSE);
+				NULL, " ", FALSE, &len);
 
 	TEST_EQ (ret, 0);
+	TEST_EQ (len, 0);
 	TEST_EQ (pos, 0);
 }
 
@@ -2064,6 +2098,144 @@ test_parse_block (void)
 	}
 }
 
+void
+test_skip_block (void)
+{
+	char      buf[1024];
+	int       ret;
+	size_t    pos, lineno, endpos;
+	NihError *err;
+
+	TEST_FUNCTION ("nih_config_skip_block");
+	program_name = "test";
+
+
+	/* Check that we can find the end of a simple block.  pos should be
+	 * updated to point past the block, and the returned endpos should
+	 * point at the end of the block itself.
+	 */
+	TEST_FEATURE ("with simple block");
+	strcpy (buf, "this is\na test\nend foo\nblah\n");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 23);
+	TEST_EQ (endpos, 15);
+
+
+	/* Check that the line number is incremented for each line that we
+	 * discover in the block, including the terminating line.
+	 */
+	TEST_FEATURE ("with line number set");
+	pos = 0;
+	lineno = 2;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, &lineno,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 23);
+	TEST_EQ (endpos, 15);
+	TEST_EQ (lineno, 5);
+
+
+	/* Check that we can find the end of a block that ends in a terminator
+	 * with extraneous whitespace around the words.
+	 */
+	TEST_FEATURE ("with whitespace in terminator");
+	strcpy (buf, "this is\na test\n  end \t foo  \nblah\n");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 29);
+	TEST_EQ (endpos, 15);
+
+
+	/* Check that we can find the end of a block that ends in a
+	 * terminator which is at the end of the file.
+	 */
+	TEST_FEATURE ("with terminator at end of file");
+	strcpy (buf, "this is\na test\nend foo");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 22);
+	TEST_EQ (endpos, 15);
+
+
+	/* Check that we can find the end of a block that ends in a
+	 * terminator which has a comment following it.
+	 */
+	TEST_FEATURE ("with terminator and comment");
+	strcpy (buf, "this is\na test\nend foo # comment\ntest\n");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 33);
+	TEST_EQ (endpos, 15);
+
+
+	/* Check that we can find the end of a block that ends in a
+	 * terminator which has a comment and then the end of file.
+	 */
+	TEST_FEATURE ("with terminator and comment at end of file");
+	strcpy (buf, "this is\na test\nend foo # comment");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 32);
+	TEST_EQ (endpos, 15);
+
+
+	/* Check that various bogus forms of terminator are ignored.
+	 */
+	TEST_FEATURE ("with various things that aren't terminators");
+	strcpy (buf, "endfoo\nend a\nend fooish\nend foo\ntest\n");
+	pos = 0;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, NULL,
+				     "foo", &endpos);
+
+	TEST_EQ (ret, 0);
+	TEST_EQ (pos, 32);
+	TEST_EQ (endpos, 24);
+
+
+	/* Check that reaching the end of the file without finding the block
+	 * terminator causes an error to be raised and NULL to be returned.
+	 */
+	TEST_FEATURE ("with no terminator before end of file");
+	strcpy (buf, "this is\na test\n");
+	pos = 0;
+	lineno = 2;
+
+	ret = nih_config_skip_block (buf, strlen (buf), &pos, &lineno,
+				     "foo", &endpos);
+
+	TEST_LT (ret, 0);
+	TEST_EQ (pos, 15);
+	TEST_EQ (lineno, 4);
+
+	err = nih_error_get ();
+	TEST_EQ (err->number, NIH_CONFIG_UNTERMINATED_BLOCK);
+	nih_free (err);
+}
+
 
 static int handler_called = 0;
 static void *last_data = NULL;
@@ -2215,7 +2387,7 @@ test_parse_stanza (void)
 
 
 	/* Check that finding an unknown stanza results in an error being
-	 * raised at its first argument, and no handler called.
+	 * raised, and no handler called.
 	 */
 	TEST_FEATURE ("with unknown stanza");
 	strcpy (buf, "wibble this is a test\nwibble\n");
@@ -2229,7 +2401,7 @@ test_parse_stanza (void)
 
 	TEST_FALSE (handler_called);
 	TEST_LT (ret, 0);
-	TEST_EQ (pos, 7);
+	TEST_EQ (pos, 0);
 	TEST_EQ (lineno, 1);
 
 	err = nih_error_get ();
@@ -2646,6 +2818,7 @@ main (int   argc,
 	test_parse_args ();
 	test_parse_command ();
 	test_parse_block ();
+	test_skip_block ();
 	test_parse_stanza ();
 	test_parse_file ();
 	test_parse ();
